window.Gleam.initSearch([{"doc":"graph","title":"graph","content":"# graph\n\n[![Package Version](https://img.shields.io/hexpm/v/graph)](https://hex.pm/packages/graph)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/graph/)\n![Supports all targets](https://img.shields.io/badge/supports-all_targets-ffaff3)\n\nA package for working with graphs in Gleam!\n\n&gt; While the package already exposes all functions you might need to implement\n&gt; complex graphs algorithms, it might still be missing some crucial functions\n&gt; to provide a pleasant developer experience working with graphs.\n&gt;\n&gt; This is still a work in progress and I&#39;m trying to figure out the best API to\n&gt; expose, so if you think something is missing, _please open an issue!_\n\n```sh\ngleam add graph\n```\n\n```gleam\nimport graph\n\npub fn main() {\n  let my_graph =\n    graph.new()\n    |&gt; graph.insert_node(1, &quot;one node&quot;)\n    |&gt; graph.insert_node(2, &quot;other node&quot;)\n    |&gt; graph.insert_directed_edge(&quot;edge label&quot;, from: 1, to: 2)\n\n  my_graph |&gt; graph.has_edge(from: 1, to: 2)\n  // -&gt; True\n\n  my_graph |&gt; graph.has_edge(from: 2, to: 1)\n  // -&gt; False\n}\n```\n","url":"index.html"},{"doc":"graph","title":"Context","content":"pub type Context(value, label) {\n  Context(\n    incoming: Dict(Int, label),\n    node: Node(value),\n    outgoing: Dict(Int, label),\n  )\n}\n The context associated with a node in a graph: it contains the node itself\n and all the incoming and outgoing edges. Edges are stored in a dict going\n from neighbour&#39;s id to the edge label.\n\nContext(\n  incoming: Dict(Int, label),\n  node: Node(value),\n  outgoing: Dict(Int, label),\n)\n\n\nSynonyms:\ngraph.Context\ngraph Context","url":"graph.html#Context"},{"doc":"graph","title":"Directed","content":"pub type Directed\n The direction of a directed graph.\n\n\nSynonyms:\ngraph.Directed\ngraph Directed","url":"graph.html#Directed"},{"doc":"graph","title":"Graph","content":"pub opaque type Graph(direction, value, label)\n A directed or undirected graph. A graph is made up of nodes and edges\n connecting them: each node holds a `value` and each edge has a `label`.\n\n The graph also carries along its `direction` (either `Directed` or\n `Undirected`) in its type so that it&#39;s impossible to mix up `Directed` and\n `Undirected` graphs inadvertently.\n\n\nSynonyms:\ngraph.Graph\ngraph Graph","url":"graph.html#Graph"},{"doc":"graph","title":"Node","content":"pub type Node(value) {\n  Node(id: Int, value: value)\n}\n A node making up a graph. Every node is identified by a number and can hold\n an arbitrary value.\n\nNode(id: Int, value: value)\n\n\nSynonyms:\ngraph.Node\ngraph Node","url":"graph.html#Node"},{"doc":"graph","title":"Undirected","content":"pub type Undirected\n The direction of an undirected graph.\n\n\nSynonyms:\ngraph.Undirected\ngraph Undirected","url":"graph.html#Undirected"},{"doc":"graph","title":"fold","content":"pub fn fold(\n  over graph: Graph(a, b, c),\n  from initial: d,\n  with fun: fn(d, Context(b, c)) -&gt; d,\n) -&gt; d\n Reduces the given graph into a single value by applying function to all its\n contexts, one after the other.\n\n &gt; 🚨 Graph&#39;s contexts are not sorted in any way so your folding function\n &gt; should never rely on any accidental order the contexts might have.\n\n ## Examples\n\n \n // The size function could be implemented using a fold.\n // The real implementation is more efficient because it doesn&#39;t have to\n // traverse all contexts!\n pub fn size(graph) {\n   fold(\n     over: graph,\n     from: 0,\n     with: fn(size, _context) { size + 1 },\n   )\n }\n \n\nSynonyms:\ngraph.fold\ngraph fold","url":"graph.html#fold"},{"doc":"graph","title":"get_context","content":"pub fn get_context(\n  graph: Graph(a, b, c),\n  of node: Int,\n) -&gt; Result(Context(b, c), Nil)\n Returns the context associated with the node with the given id, if present.\n Otherwise returns `Error(Nil)`.\n\n ## Examples\n\n \n new() |&gt; get(1)\n // -&gt; Error(Nil)\n \n\n \n new() |&gt; insert_node(Node(1, &quot;a node&quot;)) |&gt; get_context(of: 1)\n // -&gt; Ok(Context(node: Node(1, &quot;a node&quot;), ..))\n \n\nSynonyms:\ngraph.get_context\ngraph get_context","url":"graph.html#get_context"},{"doc":"graph","title":"has_edge","content":"pub fn has_edge(\n  graph: Graph(a, b, c),\n  from source: Int,\n  to destination: Int,\n) -&gt; Bool\n Returns `True` if the graph has an edge connecting the two nodes with the\n given ids.\n\n ## Examples\n\n \n let my_graph =\n   new()\n   |&gt; insert_node(Node(1, &quot;a node&quot;))\n   |&gt; insert_node(Node(2, &quot;other node&quot;))\n   |&gt; insert_directed_edge(&quot;edge label&quot;, from: 1, to: 2)\n\n my_graph |&gt; has_edge(from: 1, to: 2)\n // -&gt; True\n\n my_graph |&gt; has_edge(from: 2, to: 1)\n // -&gt; False\n \n\nSynonyms:\ngraph.has_edge\ngraph has_edge","url":"graph.html#has_edge"},{"doc":"graph","title":"has_node","content":"pub fn has_node(graph: Graph(a, b, c), node_id: Int) -&gt; Bool\n Returns `True` if the graph contains a node with the given id.\n\n ## Examples\n\n \n let my_graph = new() |&gt; insert_node(Node(1, &quot;a node&quot;))\n\n my_graph |&gt; has_node(1)\n // -&gt; True\n\n my_graph |&gt; has_node(2)\n // -&gt; False\n \n\nSynonyms:\ngraph.has_node\ngraph has_node","url":"graph.html#has_node"},{"doc":"graph","title":"insert_directed_edge","content":"pub fn insert_directed_edge(\n  graph: Graph(Directed, a, b),\n  labelled label: b,\n  from source: Int,\n  to destination: Int,\n) -&gt; Graph(Directed, a, b)\n Adds an edge connecting two nodes in a directed graph.\n\n \n let my_graph =\n   new()\n   |&gt; insert_node(Node(1, &quot;a node&quot;))\n   |&gt; insert_node(Node(2, &quot;other node&quot;))\n   |&gt; insert_directed_edge(&quot;edge label&quot;, from: 1, to: 2)\n\n my_graph |&gt; has_edge(from: 1, to: 2)\n // -&gt; True\n\n my_graph |&gt; has_edge(from: 2, to: 1)\n // -&gt; False\n \n\nSynonyms:\ngraph.insert_directed_edge\ngraph insert_directed_edge","url":"graph.html#insert_directed_edge"},{"doc":"graph","title":"insert_node","content":"pub fn insert_node(\n  graph: Graph(a, b, c),\n  node: Node(b),\n) -&gt; Graph(a, b, c)\n Adds a node to the given graph.\n If the graph already contains a node with the same id, that will be replaced\n by the new one.\n The newly added node won&#39;t be connected to any existing node.\n\n ## Examples\n\n \n new() |&gt; insert_node(Node(1, &quot;a node&quot;)) |&gt; nodes\n // -&gt; [Node(1, &quot;a node&quot;)]\n \n\nSynonyms:\ngraph.insert_node\ngraph insert_node","url":"graph.html#insert_node"},{"doc":"graph","title":"insert_undirected_edge","content":"pub fn insert_undirected_edge(\n  graph: Graph(Undirected, a, b),\n  labelled label: b,\n  between one: Int,\n  and other: Int,\n) -&gt; Graph(Undirected, a, b)\n Adds an edge connecting two nodes in an undirected graph.\n\n ## Examples\n\n \n let my_graph =\n   new()\n   |&gt; insert_node(Node(1, &quot;a node&quot;))\n   |&gt; insert_node(Node(2, &quot;other node&quot;))\n   |&gt; insert_undirected_edge(&quot;edge label&quot;, between: 1, and: 2)\n\n my_graph |&gt; has_edge(from: 1, to: 2)\n // -&gt; True\n\n my_graph |&gt; has_edge(from: 2, to: 1)\n // -&gt; True\n \nSynonyms:\ngraph.insert_undirected_edge\ngraph insert_undirected_edge","url":"graph.html#insert_undirected_edge"},{"doc":"graph","title":"map_contexts","content":"pub fn map_contexts(\n  in graph: Graph(a, b, c),\n  with fun: fn(Context(b, c)) -&gt; Context(b, c),\n) -&gt; Graph(Directed, b, c)\n Transform the contexts associated with each node.\n\n &gt; This function can add and remove arbitrary edges from the graph by\n &gt; updating the `incoming` and `outgoing` edges of a context.\n &gt; So we can&#39;t assume the final graph will still be `Undirected`, that&#39;s why\n &gt; it is always treated as a `Directed` one.\n\n ## Examples\n\n \n // The reverse function can be implemented with `map_contexts`\n pub fn reverse(graph) {\n   map_contexts(in: graph, with: fn(context) {\n     Context(\n       ..context,\n       incoming: context.outgoing,\n       outgoing: context.incoming,\n     )\n   })\n }\n \n\nSynonyms:\ngraph.map_contexts\ngraph map_contexts","url":"graph.html#map_contexts"},{"doc":"graph","title":"map_labels","content":"pub fn map_labels(\n  in graph: Graph(a, b, c),\n  with fun: fn(c) -&gt; d,\n) -&gt; Graph(a, b, d)\n Transforms the labels of all the graph&#39;s edges using the given function.\n\n ## Examples\n\n \n new()\n |&gt; insert_node(Node(1, &quot;a node&quot;))\n |&gt; insert_undirected_edge(UndirectedEdge(1, 1, &quot;label&quot;))\n |&gt; map_labels(fn(label) { label &lt;&gt; &quot;!&quot; })\n |&gt; labels\n // -&gt; [&quot;label!&quot;]\n \n\nSynonyms:\ngraph.map_labels\ngraph map_labels","url":"graph.html#map_labels"},{"doc":"graph","title":"map_values","content":"pub fn map_values(\n  in graph: Graph(a, b, c),\n  with fun: fn(b) -&gt; d,\n) -&gt; Graph(a, d, c)\n Transforms the values of all the graph&#39;s nodes using the given function.\n\n ## Examples\n\n \n new()\n |&gt; insert_node(Node(1, &quot;a node&quot;))\n |&gt; map_nodes(fn(value) { value &lt;&gt; &quot;!&quot; })\n |&gt; nodes\n // -&gt; [Node(1, &quot;my node!&quot;)]\n \n\nSynonyms:\ngraph.map_values\ngraph map_values","url":"graph.html#map_values"},{"doc":"graph","title":"match","content":"pub fn match(\n  graph: Graph(a, b, c),\n  node_id: Int,\n) -&gt; Result(#(Context(b, c), Graph(a, b, c)), Nil)\n If the graph contains a node with the given id, returns a tuple containing\n the context of that node (with all edges looping back to itself removed) and\n the &quot;remaining&quot; graph: that is, the original graph where that node has been\n removed.\n\nSynonyms:\ngraph.match\ngraph match","url":"graph.html#match"},{"doc":"graph","title":"new","content":"pub fn new() -&gt; Graph(a, b, c)\n Creates a new empty graph.\n\n ## Examples\n\n \n nodes(new())\n // -&gt; []\n \n\nSynonyms:\ngraph.new\ngraph new","url":"graph.html#new"},{"doc":"graph","title":"nodes","content":"pub fn nodes(graph: Graph(a, b, c)) -&gt; List(Node(b))\n Returns a list of all the nodes contained in the graph.\n\n ## Examples\n\n \n new() |&gt; nodes\n // -&gt; []\n \n\n \n new() |&gt; insert_node(Node(1, &quot;a node&quot;)) |&gt; nodes\n // -&gt; [Node(1, &quot;a node&quot;)]\n \n\nSynonyms:\ngraph.nodes\ngraph nodes","url":"graph.html#nodes"},{"doc":"graph","title":"remove_directed_edge","content":"pub fn remove_directed_edge(\n  graph: Graph(Directed, a, b),\n  from source: Int,\n  to destination: Int,\n) -&gt; Graph(Directed, a, b)\n Removes a directed edge connecting two nodes from a graph.\n\nSynonyms:\ngraph.remove_directed_edge\ngraph remove_directed_edge","url":"graph.html#remove_directed_edge"},{"doc":"graph","title":"remove_node","content":"pub fn remove_node(\n  graph: Graph(a, b, c),\n  node_id: Int,\n) -&gt; Graph(a, b, c)\n Removes a node with the given id from the graph. If there&#39;s no node with the\n given id it does nothing.\n\nSynonyms:\ngraph.remove_node\ngraph remove_node","url":"graph.html#remove_node"},{"doc":"graph","title":"remove_undirected_edge","content":"pub fn remove_undirected_edge(\n  graph: Graph(Undirected, a, b),\n  between one: Int,\n  and other: Int,\n) -&gt; Graph(Undirected, a, b)\n Removes an undirected edge connecting two nodes from a graph.\n\nSynonyms:\ngraph.remove_undirected_edge\ngraph remove_undirected_edge","url":"graph.html#remove_undirected_edge"},{"doc":"graph","title":"reverse_edges","content":"pub fn reverse_edges(\n  graph: Graph(Directed, a, b),\n) -&gt; Graph(Directed, a, b)\n Flips the direction of every edge in the graph. All incoming edges will\n become outgoing and vice-versa.\n\nSynonyms:\ngraph.reverse_edges\ngraph reverse_edges","url":"graph.html#reverse_edges"},{"doc":"graph","title":"size","content":"pub fn size(graph: Graph(a, b, c)) -&gt; Int\n Returns the number of nodes of the graph.\n\n ## Examples\n\n \n new() |&gt; size\n // -&gt; 0\n \n\n \n new() |&gt; insert_node(Node(1, &quot;a node&quot;)) |&gt; size\n // -&gt; 1\n \n\nSynonyms:\ngraph.size\ngraph size","url":"graph.html#size"},{"doc":"graph","title":"to_directed","content":"pub fn to_directed(\n  graph: Graph(Undirected, a, b),\n) -&gt; Graph(Directed, a, b)\n Turns an undirected graph into a directed one. Every edge connecting two\n nodes in the original graph will be considered as a pair of edges connecting\n the nodes going in both directions.\n\nSynonyms:\ngraph.to_directed\ngraph to_directed","url":"graph.html#to_directed"},{"doc":"graph","title":"graph","content":"\n Here&#39;s a handy index you can use to browse through the various graph\n functions.\n\n | operation kind | functions |\n |---|---|\n | creating graphs | [`new`](#new) |\n | turning graphs into lists | [`nodes`](#nodes) |\n | querying a graph | [`size`](#size), [`has_node`](#has_node), [`has_edge`](#has_edge), [`get_context`](#get_context), [`match`](#match) |\n | adding/removing elements from a graph | [`insert_node`](#insert_node), [`insert_directed_edge`](#insert_directed_edge), [`insert_undirected_edge`](#insert_undirected_edge), [`remove_node`](#remove_node), [`remove_directed_edge`](#remove_directed_edge), [`remove_undirected_edge`](#remove_undirected_edge) |\n | transforming graphs | [`fold`](#fold), [`reverse`](#reverse), [`map_contexts`](#map_contexts), [`map_values`](#map_values), [`map_labels`](#map_labels), [`reverse_edges`](#reverse_edges), [`to_directed`](#to_directed) |\n","url":"graph.html"}]);